import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;



// a set of buttons for the player to do various game-related actions with.
public class CGActionButtonSet extends Composite {

    // the labels and tooltips for the Buttons:
	// 'Swap Letters': swap the LetterSet, 
	// 'Cancel Word': empty the board of unfixed letters,
	// 'Submit Word': submit inserted word to opponent,
    // 'Connect': connect to opponent.
	final private static String SWAP_LETTERS	= "Kirjainvaihto";
	final private static String SWAP_LETTERS_TT	= "Vaihtaa kaikki pelaajan kirjaimet uusiin.";
	final private static String CANCEL_WORD		= "Peruuta";
	final private static String CANCEL_WORD_TT	= "Poistaa irtokirjaimet pelilaudalta.";
	final private static String SUBMIT_WORD		= "Kiinnit";
	final private static String SUBMIT_WORD_TT	= "Kiinnitt irtokirjaimet pelilaudalle.";
	final private static String CONNECT			= "Yhdist";
	final private static String CONNECT_TT		= "Muodostaa yhteyden vastapelaajaan.";
	final private static String DISCONNECT		= "Katkaise";
	final private static String DISCONNECT_TT	= "Katkaisee yhteyden vastapelaajaan.";
	
	// disconnect confirmation dialog's contents.
	final private static String DISCONNECT_CONFIRMATION_TITLE	 = "Varmistus: yhteyden katkaisu";
	final private static String DISCONNECT_CONFIRMATION_MESSAGE =
		"Yhteyden katkaisu merkitsee sit, ett\n" +
		"yhteys vastapelaajaan menetetn ja\n" +
		"meneilln oleva peli keskeytyy.\n" +
		"\n" +
		"Oletko varma ett haluat katkaista yhteyden?";
	
	final private Button swapLetters;
    final private Button cancelWord;
    final private Button submitWord;
    final private Button connect;

    // whether attempting to connect || already connected or not.
    // CGConnection needs to see this.
    public boolean connecting;
    
	// constructor sets the parent window and game.
	final CrosswordGame game;
	final Shell shell;
	public CGActionButtonSet(final Shell s, final CrosswordGame g) {
		super(s, SWT.NONE);
		shell = s;
		game = g;

		GridLayout gl = new GridLayout(5, true);
        GridData fill = new GridData(GridData.FILL_HORIZONTAL);
		this.setLayout(gl);
		this.setLayoutData(fill);

        ActionButtonListener listener = new ActionButtonListener();

        // the Button for swapping the LetterSet.
        swapLetters = new Button(this, SWT.PUSH);
        swapLetters.setText(SWAP_LETTERS);
        swapLetters.setToolTipText(SWAP_LETTERS_TT);
        swapLetters.setLayoutData(fill);
		swapLetters.addListener(SWT.Selection, listener);
        
        // the Button for clearing unfixed letters back into the player's LetterSet.
        cancelWord = new Button(this, SWT.PUSH);
        cancelWord.setText(CANCEL_WORD);
        cancelWord.setToolTipText(CANCEL_WORD_TT);
        cancelWord.setLayoutData(fill);
		cancelWord.addListener(SWT.Selection, listener);
		
        // the Button for fixing a laid out word and passing turn to the opponent.
        submitWord = new Button(this, SWT.PUSH);
        submitWord.setText(SUBMIT_WORD);
        submitWord.setToolTipText(SUBMIT_WORD_TT);
        submitWord.setLayoutData(fill);
		submitWord.addListener(SWT.Selection, listener);
		
		// space between 'Connect' and other Buttons.
		new Label(this, SWT.NONE).setLayoutData(fill);
		
        // the Button for connecting to an opponent.
        connect = new Button(this, SWT.PUSH);
        connect.setText(CONNECT);
        connect.setToolTipText(CONNECT_TT);
        connect.setLayoutData(fill);
		connect.addListener(SWT.Selection, listener);
		
		connecting = false;
	}
	
	// disable/enable all the Buttons.
	public void disable() {
		swapLetters.setEnabled(false);
		cancelWord.setEnabled(false);
		submitWord.setEnabled(false);
		//connect.setEnabled(false);
	}
	public void enable() {
		// leave 'Swap' disabled if out of alphabet.
		if (game.alphabet.size() > 0)	swapLetters.setEnabled(true);
		cancelWord.setEnabled(true);
		submitWord.setEnabled(true);
		//connect.setEnabled(true);
	}
	
	// enable 'Connect' & disable 'Disconnect'.
	// CGConnection and CGWaiter need to know about these.
	public void connectToDisconnect() {
		connect.setText(DISCONNECT);
		connect.setToolTipText(DISCONNECT_TT);
	}
	// disable 'Connect & enable 'Disconnect'.
	public void disconnectToConnect() {
		connect.setText(CONNECT);
		connect.setToolTipText(CONNECT_TT);
	}
	
	// disable/enable the 'Swap Letters' Button.
	// CGWaiter and CrosswordGame need to know about these.
	public void disableSwap() {
		swapLetters.setEnabled(false);
	}
	public void enableSwap() {
		swapLetters.setEnabled(true);
	}

	// the Button selection listener.
	private class ActionButtonListener implements Listener {

		// data for the confirmation dialog presented when
		// the player selects the 'Swap Letters' Button.
		// the _CONNECTED version is for when there is a connection to an opponent.
		final private static String SWAP_CONFIRMATION_TITLE =
			"Varmistus: kirjainten vaihtaminen";
		final private static String SWAP_CONFIRMATION_PROMPT_CONNECTED =
			"Kirjainten vaihtaminen merkitsee sit, ett\n" +
			"- pelilaudalle tll vuorolla syttmsi\n" +
			"  kirjaimet palautetaan kirjaimistoosi\n" +
			"- kirjaimistoosi arvotaan uudet kirjaimet\n" +
			"- vuoro siirtyy vastapelaajalle\n" +
			"- jt tll kierroksella vaille pisteit.\n" +
			"\n" +
			"Oletko varma ett haluat vaihtaa kirjaimet?";
		final private static String SWAP_CONFIRMATION_PROMPT_UNCONNECTED =
			"Kirjainten vaihtaminen merkitsee sit, ett\n" +
			"- pelilaudalle tll vuorolla syttmsi\n" +
			"  kirjaimet palautetaan kirjaimistoosi\n" +
			"- kirjaimistoosi arvotaan uudet kirjaimet.\n" +
			"\n" +
			"Oletko varma ett haluat vaihtaa kirjaimet?";
		
		// the status report after swapping the letters.
		final private static String STATUS_SWAP_LETTERS_SUCCESS	= "Kirjaimet vaihdettu.";

	    // spring to action when a button is clicked.
	    public void handleEvent(Event e) {

	    	// grab the selected button.
	    	Button button = (Button)e.widget;

	    	// 'Swap Letters': replace player's current letters
	    	// with a new ones from the alphabet.
	    	// identify given Button, for the listener.
	    	if (button == swapLetters) {

	    		// request confirmation from the user,
	    		// unless not connected to opponent
	    		// and no new letters have been laid on the board.
	    		if (!(!game.connection.connected && game.crossword.countNew() == 0)) {
					if (
							!(game.confirmed(
								SWAP_CONFIRMATION_TITLE,
								SWAP_CONFIRMATION_PROMPT_UNCONNECTED,
								SWAP_CONFIRMATION_PROMPT_CONNECTED)
						)
					)	return;
	    		}
	    		
				// clean up all unfixed letters from board.
	    		game.crossword.cancelWords(true);
	    		
	    		// push player's letters back into the Alphabet.
	    		game.alphabet.pushBackAll();
	    		
				// reset score to what it was prior to this turn.
				game.scoreTable.restoreScore();
				
				// if connected to opponent,
				// send the move, finish it at this end and wait for turn.
				if (game.connection.connected) {
					
					// this feels like bending the rules... this constructor is
					// used mostly at the opposite end, but it works here just as well.
					CGMove move = new CGMove(game, CGProtocol.MOVE_TYPES_SWAP + game.alphabet.toString());
					if ( !(game.connection.sendMove(move)) ) game.disconnected();
					
		    		// pick new letters.
		    		// this has to be done after the newly scrambled Alphabet have been sent
					// so that it can be repeated at the opponent's end, and so as not to
					// have to send the picked letters separately.
					game.playerSlots.initialize();
					
					game.connection.waitForTurn();
					return;
				}
				
	    		// pick new letters.
				game.playerSlots.initialize();
				
				game.setStatus(STATUS_SWAP_LETTERS_SUCCESS);
	    	}
	    	
	    	// 'Cancel Word': clean up all unfixed letters from board.
	    	else if (button == cancelWord) {
	    		game.crossword.cancelWords();
	    		
				// if connected to opponent,
				// send the move.
				if (game.connection.connected) {
					CGMove move = new CGMove(game, null, null, CGProtocol.MOVE_TYPES_CANCEL);
					if ( !(game.connection.sendMove(move)) ) game.disconnected();
				}
	    	}

	    	// 'Submit Word': fix the inserted word, increase score,
	    	// pass the turn to the opponent.
	    	else if (button == submitWord) {
	    		
	    		// the number of points to add to player's score for the move.
	    		// initialize to a smart evaluation.
				int ptsForMove = game.crossword.smartScore();
				
				// read the inserted word.
				String newWord = game.crossword.getInsertedWord();
				
				// a special case: no word inserted.
				if ( game.crossword.countNew() == 0 ) {
	    			if (new CGSubmitDialog(shell, game).open() == CGSubmitDialog.SCORE_CANCEL)
	    				return;
				}
				
				else {
					if ( game.scoreTable.getLastScore() != game.scoreTable.getPlayerScore()
							&& game.dictionary.contains(newWord) ) {
						ptsForMove = 0;
					}
					else {
		    			CGSubmitDialog getScore = new CGSubmitDialog(ptsForMove, newWord, shell, game);
		    			ptsForMove = getScore.open();
		    			
		    			// if the player selected 'Cancel', cancel. 
		    			if (ptsForMove == CGSubmitDialog.SCORE_CANCEL)		return;
					}
	    		}
				
	        	// fix all letters on board,
	        	// fill all empty letter slots with letters.
	    		// add points to player's score.
	    		game.crossword.fixWords();
	    		game.playerSlots.fill();
	    		game.scoreTable.increasePlayerScore(ptsForMove);
	    		
	    		// disable 'Swap Letters' if out of alphabet.
	    		if ( !(game.alphabet.size() > 0) )	disableSwap();

	    		// the opponent needs to be sent the score difference set by the player.
	    		if (ptsForMove == 0)
	    			ptsForMove = game.scoreTable.getPlayerScore() - game.scoreTable.getLastScore();
	    		
	    		// record the current score and points given for this move.
	    		game.scoreTable.recordScore(ptsForMove);
	    		
	    		// update alphabet counter.
	    		game.scoreTable.setAlphabetCounter();
	    		
				// if connected to opponent,
				// send the move.
				if (game.connection.connected) {
					CGMove move = new CGMove(game, CGProtocol.MOVE_TYPES_FIX, ptsForMove);
					game.connection.sendMove(move);
					game.connection.waitForTurn();
				}
	    	}

	    	// 'Connect':
			// connect to an opponent.
	    	else if ( (button == connect) && connecting ) {
				if (game.connection.connected) {
					if	(game.confirmed(
								DISCONNECT_CONFIRMATION_TITLE,
								DISCONNECT_CONFIRMATION_MESSAGE
						)
					) 		game.disconnected(CrosswordGame.DISCONNECTED);
					else	return;
				}
				else	game.disconnected("");
				connecting = false;
	    	}
		    else if ( (button == connect) && !connecting ) {
		    	
	    		connecting = true;

	    		int goAhead = ContinueDialog.NEWGAME;
	    		
	    		// if moves have been made, query the user whether (s)he wants to
	    		// - continue the current game with the opponent after connecting
	    		// - begin a new game with the opponent after connecting
	    		// - not to connect at all (cancel).
	    		if (!game.noMovesMade())	goAhead = new ContinueDialog(shell).open();
	    		
				switch (goAhead) {
					case (ContinueDialog.CONTINUE) :
						game.connection.connect(true);
						break;
						
					case (ContinueDialog.NEWGAME) :
			    		game.initialize(true);
						game.connection.connect();
						break;
						
					case (ContinueDialog.CANCEL) :
						return;
				}
	    	}
	    }

	    
	    
		// queries the user whether (s)he wants to
		// - continue the current game with the opponent after connecting
		// - begin a new game with the opponent after connecting
		// - not to connect at all (cancel).
	    private class ContinueDialog extends Dialog {
	    	
	    	// the dialog title, explanation of the alternatives and a final prompt.
	    	final private static String TITLE	= "Varmistus: yhteyden muodostaminen";
	    	final private static String EXPLANATION	=
	    		"Halutessasi voit lhett tmnhetkisen pelitilanteen vastustajalle, " +
	    		"jolloin voitte jatkaa mahdollisesti aiemmin kesken jnytt peli. " +
	    		"Voit toisaalta valita mys uuden pelin aloittamisen, " +
	    		"jolloin nykyinen pelitilanne hvitetn ja pelaajille arvotaan uudet kirjaimet. " +
	    		"Voit viel mys perua yhteyden muodostamisen.";
	    	final private static String PROMPT	=
	    		"Haluatko vahvistaa yll tekemsi valinnan ja yritt yhteydenmuodostusta?";

	    	// the dialog width.
	    	final private int WIDTH	= 200;
	    	
	    	// the selection Buttons, their labels and tooltips.
	    	final private static String CONTINUE_LABEL	= "Jatketaan nykyist peli";
	    	final private static String NEWGAME_LABEL	= "Aloitetaan uusi peli";
	    	final private static String PROCEED_LABEL	= "Kyll";
	    	final private static String CANCEL_LABEL	= "Ei";
	    	private Button proceed;
	    	private Button cancel;
	    	
	    	// identifiers for the buttons, for use by the parent buttonset.
	    	final public static int CONTINUE	= 1;
	    	final public static int NEWGAME		= 2;
	    	final public static int CANCEL		= 0;

	    	// the dialog window. 
	    	private Shell shell;

	    	// the selection, to be returned.
	    	int selection;
	    	
	    	// the two choices (continue current game/start a new game).
	    	private RadioSelector selector;
	    	
	    	public ContinueDialog(final Shell parent) {
	    		super (parent, SWT.NONE);
	    	}
	    	
	    	public int open() {
	    		
	    		Shell parent = getParent();
	    		shell = new Shell(parent, SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL);
	    		GridLayout gl = new GridLayout(2, true); 
	    		gl.marginHeight = 20;
	    		gl.marginWidth = 20;
	    		shell.setLayout(gl);
	    		shell.setText(TITLE);

	    		// explanation of the alternatives.
	    		final Label explanation = new Label(shell, SWT.WRAP);
	    		explanation.setText(EXPLANATION);
	    		GridData gdexp = new GridData();
	    		gdexp.horizontalSpan = 2;
	    		gdexp.widthHint = WIDTH;
	    		explanation.setLayoutData(gdexp);

		    	// the two choices (continue current game/start a new game).
	    		selector = new RadioSelector(shell);
	    		GridData gdselector = new GridData();
	    		gdselector.horizontalSpan = 2;
	    		gdselector.widthHint = WIDTH;
	    		selector.setLayoutData(gdselector);
	    		
	    		// the final prompt.
	    		final Label prompt = new Label(shell, SWT.WRAP);
	    		prompt.setText(PROMPT);
	    		GridData gdprompt = new GridData();
	    		gdprompt.horizontalSpan = 2;
	    		gdprompt.widthHint = WIDTH;
	    		prompt.setLayoutData(gdprompt);
	    		
	    		// the confirmation/cancellation Buttons.
	    		proceed = new Button(shell, SWT.PUSH);
	    		proceed.setText(PROCEED_LABEL);
	    		GridData gdbuttons = new GridData(GridData.FILL_BOTH);
	    		proceed.setLayoutData(gdbuttons);

	    		cancel = new Button(shell, SWT.PUSH);
	    		cancel.setText(CANCEL_LABEL);
	    		GridData gdbuttons2 = new GridData(GridData.FILL_BOTH);
	    		cancel.setLayoutData(gdbuttons2);
	    		
	    		ButtonListener listener = new ButtonListener();
	    		proceed.addListener(SWT.Selection, listener);
	    		cancel.addListener(SWT.Selection, listener);

	    	    shell.pack();
	    		shell.open();

	    		Display display = parent.getDisplay();
	    		while (!shell.isDisposed()) {
	    			if (!display.readAndDispatch()) display.sleep();
	    		}
	    		
	    		return selection;
	    	}
	    	
	    	
	    	
	    	// the two choices (continue current game/start a new game).
	    	private class RadioSelector extends Composite {
	    		
	    		// this needs to be seen by continueSelected().
	    		// ('continue' is a reserved word, therefore deliberately misspelled.)
	    		private Button contnue;
	    		
	    		public RadioSelector(final Shell parent) {
	    			super(parent, SWT.BORDER);
	    			GridLayout gl = new GridLayout(1, true); 
	    			gl.marginHeight = 10;
	    			gl.marginWidth = 25;
	    			this.setLayout(gl);
	    			
	    			GridData gdradio = new GridData();
	    			gdradio.widthHint = WIDTH;
	    			
	    			contnue = new Button(this, SWT.RADIO);
	    			contnue.setText(CONTINUE_LABEL);
	    			contnue.setLayoutData(gdradio);

	    			Button newgame = new Button(this, SWT.RADIO);
	    			newgame.setText(NEWGAME_LABEL);
	    			newgame.setLayoutData(gdradio);
	    		}
	    		public boolean continueSelected() {
	    			return contnue.getSelection();
	    		}
	    	}
	    	
	    	// react to confirmation/cancellation.
	    	private class ButtonListener implements Listener {
	    		public void handleEvent(final Event e) {
	    			if ((Button)e.widget == proceed) {
	    				if (selector.continueSelected())selection = CONTINUE;
	    				else							selection = NEWGAME;
	    			}
	    			else								selection = CANCEL;
	    			shell.dispose();
	    	    }
	    	}
	    }
	}
}
