import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.VerifyEvent;
import org.eclipse.swt.events.VerifyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;



// query the player for his opponent's IP address.
public class CGAddressDialog extends Dialog {
	
	// other classes need not know about any of the fields.

	// the dialog title and the prompt above the IP input field.
	final private static String TITLE	= "Vastapelaajan IP-osoite";
	final private static String PROMPT	= "Anna vastapelaajan IP-osoite:";
	
	// the Buttons below the IP input field and their labels.
	final private static String APPLY	= "Kyt";
	final private static String SAVE	= "Tallenna";
	final private static String EXIT	= "Takaisin peliin";
	private Button apply;
	private Button save;
	private Button exit;

	// the IP input field.
	private CGOpponentIP ipField;
	
	// the dialog window. 
	private Shell shell;

	
	
	// the constructor sets the parent game.
	private final CrosswordGame game;
	public CGAddressDialog(final Shell parent, final CrosswordGame g) {
		super (parent, SWT.NONE);
		game = g;
	}

	public void open() {
		
		Shell parent = getParent();
		shell = new Shell(parent, SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL);
		GridLayout gl = new GridLayout(3, true); 
		gl.marginHeight = 20;
		gl.marginWidth = 20;
		shell.setLayout(gl);
		shell.setText(TITLE);

		// set the prompt above the IP field.
		final Label prompt = new Label(shell, SWT.CENTER);
		prompt.setText(PROMPT);
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 3;
		prompt.setLayoutData(gd);
	
		// the IP input field.
		ipField = new CGOpponentIP();
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 3;
		gd.horizontalAlignment = SWT.CENTER;
		ipField.setLayoutData(gd);
	    ipField.setIP(game.settings.getOpponentIP());

		// the choice Buttons: 'Apply', 'Save', 'Exit'.
		apply = new Button(shell, SWT.PUSH);
		apply.setText(APPLY);
		apply.setEnabled(false);

		save = new Button(shell, SWT.PUSH);
		save.setText(SAVE);
		save.setEnabled(false);

		exit = new Button(shell, SWT.PUSH);
		exit.setText(EXIT);
		
		GridData gdbuttons = new GridData(GridData.FILL_BOTH);
		apply.setLayoutData(gdbuttons);
		save.setLayoutData(gdbuttons);
		exit.setLayoutData(gdbuttons);
		
		ButtonListener listener = new ButtonListener();
		apply.addListener(SWT.Selection, listener);
		save.addListener(SWT.Selection, listener);
		exit.addListener(SWT.Selection, listener);

	    shell.pack();
		shell.open();

		Display display = parent.getDisplay();
		while (!shell.isDisposed()) {
			if (!display.readAndDispatch()) display.sleep();
		}
	}
	
	// disable/enable the 'Apply' and 'Save' buttons.
	private void disableApply() {
		apply.setEnabled(false);
	}
	private void enableApply() {
		apply.setEnabled(true);
	}
	private void disableSave() {
		save.setEnabled(false);
	}
	private void enableSave() {
		save.setEnabled(true);
	}
	
	
	
	// react to buttons being clicked.
	// 'Apply' and 'Save' assume the entered data is valid.
	// it should be, since they're disabled when it's not.
	private class ButtonListener implements Listener {

		public void handleEvent(Event e) {

			final Button clicked = (Button)e.widget;
			
			if (clicked == apply) {
				game.connection.setOpponentIP(ipField.getIP());
				apply.setEnabled(false);
			}
			
			else if (clicked == save) {
				game.connection.setOpponentIP(ipField.getIP());
				apply.setEnabled(false);
				
				game.settings.setOpponentIP(ipField.getIP());
				game.settings.save();
				save.setEnabled(false);
			}
			
			else {
				shell.dispose();
			}
	    }
	}
	
	
	
	// the IP address input field.
	// the listener was adapted from
	// http://www.java2s.com/Code/Java/SWT-JFace-Eclipse/Demonstratesvariouslisteners.htm
	private class CGOpponentIP extends Composite implements VerifyListener, ModifyListener {
		
		// the input mechanism is a set of four Text widgets.
		final private Text ip1;
		final private Text ip2;
		final private Text ip3;
		final private Text ip4;
		
		// the constructor sets initializes the input to a default value.
		public CGOpponentIP() {
			super(shell, 0);
			this.setLayout(new GridLayout(7, false));
			GridData ipf = new GridData();
			ipf.widthHint = 21;
			
			ip1 = new Text(this, SWT.SINGLE | SWT.BORDER);
			ip1.setTextLimit(3);
			ip1.setLayoutData(ipf);
			new Label(this, SWT.NONE).setText(".");
			ip2 = new Text(this, SWT.SINGLE | SWT.BORDER);
			ip2.setTextLimit(3);
			ip2.setLayoutData(ipf);
			new Label(this, SWT.NONE).setText(".");
			ip3 = new Text(this, SWT.SINGLE | SWT.BORDER);
			ip3.setTextLimit(3);
			ip3.setLayoutData(ipf);
			new Label(this, SWT.NONE).setText(".");
			ip4 = new Text(this, SWT.SINGLE | SWT.BORDER);
			ip4.setTextLimit(3);
			ip4.setLayoutData(ipf);
			
			setIP(CGSettings.DEFAULT_OPPONENT_IP);
		}
		
		// spring to action when a character is typed.
		public void verifyText(VerifyEvent e) {
			
			// allow only backspace, delete and digits.
			// http://www.java2s.com/Tutorial/Java/0280__SWT/Allowbackspaceanddelete.htm
			char typedCharacter = e.character;
			e.doit = false;
			if (typedCharacter == '\u0008' || typedCharacter == '\u007F')	e.doit = true;
			else if (Character.isDigit(typedCharacter))						e.doit = true;
		}

		// spring to action after text in one of the widgets is changed.
		public void modifyText(ModifyEvent event) {

			// if entered data constitutes a valid IP address,
			// enable 'Apply' and 'Save', otherwise disable them.
			if (isValid(getIP())) {
				enableApply();
				enableSave();
			}
			
			else {
				disableApply();
				disableSave();
			}
		}

		// get the currently entered IP.
		public String getIP() {
			return
				stripZeros(ip1.getText()) + "." + stripZeros(ip2.getText()) + "." +
				stripZeros(ip3.getText()) + "." + stripZeros(ip4.getText());
		}

		// strip any leading zeroes from given String.
		private String stripZeros(String s) {
			String stripped;
			try {
				stripped = Integer.toString(Integer.parseInt(s));
				return stripped;
			}
			catch (NumberFormatException ex) {
				return s;
			}
		}
		
		// convert a String to a natural number (>= 0).
		// returns a negative if given String is not a natural number.
		private int toNumber(String s) {
			int i;
		    try {
		    	i = Integer.parseInt(s);
		    }
		    
		    catch (NumberFormatException ex) {
		    	i = -1;
		    }
		    return i;
		}
		
		// test whether given String is a valid IP address.
		// (here "valid" means its of the form "N.N.N.N",
		// where 0 <= N <= 255.)
		private boolean isValid(String sIp) {

			// if String is shorter than the shortest IP address
			// such as a "0.0.0.0".
			if (sIp.length() < 7) return false;
			
			// split the given String at dots,
			// analyse each dot-separated part individually.
			int dot, i;
			int parts = 0;
			while ( (dot = sIp.indexOf(".")) > 0) {

				parts++;
				if (parts>4) 				return false;
				
				// if this isn't a valid part of an IP address.
				i = toNumber(sIp.substring(0, dot));
				if ( (i < 0) || (i > 255) ) return false;
				
				// if the String ends in a dot.
				if (sIp.length() < (dot+2)) return false;
				
				sIp = sIp.substring(dot+1);
			}

			// pick and test the last part, which wasn't limited by a dot.
			parts++;
			if (parts != 4) return false;
			i = toNumber(sIp);
			if ( (i < 0) || (i > 255) ) return false;
			
			// reaching here ensures the String qualifies as an IP address.
			return true;
		}
		
		// set the IP address according to given String.
		public void setIP (String sIp) {
			
			// the modification listeners need to be removed
			// to be able to change the Text widgets' contents.
			ip1.removeVerifyListener(this);
			ip2.removeVerifyListener(this);
			ip3.removeVerifyListener(this);
			ip4.removeVerifyListener(this);
		    ip1.removeModifyListener(this);
		    ip2.removeModifyListener(this);
		    ip3.removeModifyListener(this);
		    ip4.removeModifyListener(this);

			// default to localhost on a serious error.
			if (!isValid(sIp)) {
				System.err.println("DEBUG: AddressDialog: Ohjelma yritti asettaa " +
						"virheellisen IP-osoitteen '" + sIp + "'.");
				sIp = "127.0.0.1";
				System.err.println("DEBUG: AddressDialog: Turvaudutaan oletusosoitteeseen " +
						sIp + ".");
			}
			// the given String is now valid,
			// so it can be split at dots.
			
			int dot;
			String sIp1, sIp2, sIp3, sIp4;
			
			dot = sIp.indexOf(".");
			sIp1 = sIp.substring(0, dot);
			sIp = sIp.substring(dot+1);
			dot = sIp.indexOf(".");
			sIp2 = sIp.substring(0, dot);
			sIp = sIp.substring(dot+1);
			dot = sIp.indexOf(".");
			sIp3 = sIp.substring(0, dot);
			sIp4 = sIp.substring(dot+1);
			
			// set the Text widgets.
			ip1.setText(sIp1);
			ip2.setText(sIp2);
			ip3.setText(sIp3);
			ip4.setText(sIp4);

			// add the listeners back.
			ip1.addVerifyListener(this);
			ip2.addVerifyListener(this);
			ip3.addVerifyListener(this);
			ip4.addVerifyListener(this);
		    ip1.addModifyListener(this);
		    ip2.addModifyListener(this);
		    ip3.addModifyListener(this);
		    ip4.addModifyListener(this);
		}
	}
}
