import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;


public class CGCell extends CGLetterSlot {
	
	// the default background color of a Button,
	// and the highlighting colors for newly-inserted and fixed letters,
	// and one for the center cell.
	final private static Color BACKGROUND_COLOR_DEFAULT	= Display.getDefault().getSystemColor(SWT.COLOR_WIDGET_BACKGROUND);
	final private static Color BACKGROUND_COLOR_UNFIXED	= Display.getDefault().getSystemColor(SWT.COLOR_GREEN);
	final private static Color BACKGROUND_COLOR_NEWFIXED= Display.getDefault().getSystemColor(SWT.COLOR_DARK_GREEN);
	final private static Color BACKGROUND_COLOR_CENTER	= Display.getDefault().getSystemColor(SWT.COLOR_RED);
	
	// the coordinates associated with the cell.
	final public int x;
	final public int y;
	
	public boolean fixed;
	
	public CGCell(final int xInit, final int yInit, final Composite parent, final CrosswordGame g) {
		super(parent, g);
		x = xInit;
		y = yInit;
	}
	public void initialize() {
		free();
		super.initialize();
	}
	// highlight the cell as containing an unfixed letter.
	public void highlight() {
		button.setBackground(BACKGROUND_COLOR_UNFIXED);
	}
	// restore the cell as containing no letters.
	public void unHighlight() {
		button.setBackground(BACKGROUND_COLOR_DEFAULT);
	}
	
	// color the cell as the center.
	public void setAsCenter() {
		this.setBackground(BACKGROUND_COLOR_CENTER);
	}
	
	// color the cell as newly-fixed.
	public void setAsNew() {
		button.setBackground(BACKGROUND_COLOR_NEWFIXED);
	}
	
	// color the cell as old fixed.
	// CGCorssword needs to see this.
	public void setAsOld() {
		button.setBackground(BACKGROUND_COLOR_DEFAULT);
	}
	
	// fix the cell, i.e. make it unchangeable.
	// return TRUE if the cell was previously unfixed,
	// return FALSE if the cell was already fixed.
	public boolean fix() {
		setAsOld();
		if (fixed) {
			fixed = true;
			return false;
		}
		else {
			fixed = true;
			return true;
		}
	}
	// clear the cell and make it changeable.
	// return TRUE if the cell was previously fixed,
	// return FALSE if the cell was already free.
	public boolean free() {
		unHighlight();
		if (fixed) {
			fixed = false;
			return true;
		}
		else {
			fixed = false;
			return false;
		}
	}
	// letter pickup / swap with queue.
	public void widgetSelected(SelectionEvent e) {	widgetDefaultSelected(e);	}
	public void widgetDefaultSelected(SelectionEvent e) {
		if (!fixed) super.widgetDefaultSelected(e);
	}
}
