import org.eclipse.swt.SWT;
import org.eclipse.swt.SWTException;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.Shell;



//the menus of the main window. 
public class CGMenu {

	// the menus and their items.
	final private static String MENU_GAME								= "&Peli";
	final private static String MENU_GAME_ITEM_DICTIONARY				=	"Sanas&to";
	final private static String MENU_GAME_ITEM_UNDO						=	"Ku&moa";
	final private static String MENU_GAME_ITEM_RESET					=	"&Nollaa";
	final private static String MENU_GAME_ITEM_EXIT						=	"&Lopeta";
	final private static String MENU_SETTINGS							= "A&setukset";
	final private static String MENU_SETTINGS_MENU_PLAYER				=	"Pel&aaja";
	final private static String MENU_SETTINGS_MENU_PLAYER_ITEM_PLAYER1	=		"Jan&i";
	final private static String MENU_SETTINGS_MENU_PLAYER_ITEM_PLAYER2	=		"Jann&e";
	final private static String MENU_SETTINGS_ITEM_ADDRESS				=	"&Vastustajan IP-osoite";
	
	// the menu bar itself. the disabler/enabler need to see it.
	private Menu menuBar;

	// the Game > Exit confirmation dialog's contents.
	// which of the two messages is displayed is determined by
	// whether a connection is up when the confirmation is represented.
	final private static String EXIT_CONFIRMATION_TITLE	 = "Varmistus: pelin lopettaminen";
	final private static String EXIT_CONFIRMATION_MESSAGE_CONNECTED =
		"Pelin lopettaminen merkitsee sit, ett\n" +
		"yhteys vastapelaajaan katkaistaan ja\n" +
		"tiedot pelilaudalla olevista kirjaimista\n" +
		"menetetn.\n" +
		"\n" +
		"Oletko varma ett haluat lopettaa pelin?";
	final private static String EXIT_CONFIRMATION_MESSAGE =
		"Pelin lopettaminen merkitsee sit, ett\n" +
		"tiedot pelilaudalla olevista kirjaimista\n" +
		"menetetn.\n" +
		"\n" +
		"Oletko varma ett haluat lopettaa pelin?";

	// the undo item needs to be seen by its disabler/enabler.
	final private MenuItem undo;
	
	// the player selector items need to be seen by their listener.
    final private MenuItem player1;
    final private MenuItem player2;

	// the constructor parameters are the main window and the parent game.
	final private CrosswordGame game;
	final private Shell shell;
	public CGMenu(final Shell s, final CrosswordGame g) {
		shell = s;
		game = g;
		
		menuBar = new Menu(shell, SWT.BAR);

		// Game
		// - Dictionary
		// - Undo
		// - Reset
	    // `- Exit
		MenuItem gameHeader = new MenuItem(menuBar, SWT.CASCADE);
		gameHeader.setText(MENU_GAME);
		Menu gameMenu = new Menu(shell, SWT.DROP_DOWN);
		gameHeader.setMenu(gameMenu);
		
	    final MenuItem dictionary = new MenuItem(gameMenu, SWT.PUSH);
	    dictionary.setText(MENU_GAME_ITEM_DICTIONARY);
	    dictionary.addSelectionListener(new dictionaryListener());
	    
	    undo = new MenuItem(gameMenu, SWT.PUSH);
	    undo.setText(MENU_GAME_ITEM_UNDO);
	    undo.addSelectionListener(new undoListener());
	    
	    new MenuItem(gameMenu, SWT.SEPARATOR);

	    final MenuItem reset = new MenuItem(gameMenu, SWT.PUSH);
	    reset.setText(MENU_GAME_ITEM_RESET);
	    reset.addSelectionListener(new resetListener());
	    
	    final MenuItem exit = new MenuItem(gameMenu, SWT.PUSH);
	    exit.setText(MENU_GAME_ITEM_EXIT);
	    exit.addSelectionListener(new exitListener());
	    
	    // Settings
		// `- Player
	    //    `- Player 1
	    //    `- Player 2
	    // `- Opponent's address

		MenuItem settingsHeader = new MenuItem(menuBar, SWT.CASCADE);
		settingsHeader.setText(MENU_SETTINGS);
		Menu settingsMenu = new Menu(shell, SWT.DROP_DOWN);
		settingsHeader.setMenu(settingsMenu);

		final MenuItem player = new MenuItem(settingsMenu, SWT.CASCADE);
	    player.setText(MENU_SETTINGS_MENU_PLAYER);
	    final Menu playerMenu = new Menu(shell, SWT.DROP_DOWN);
	    player.setMenu(playerMenu);
	    
	    player1 = new MenuItem(playerMenu, SWT.RADIO);
	    player1.setText(MENU_SETTINGS_MENU_PLAYER_ITEM_PLAYER1);
	    player1.addSelectionListener(new playerListener());
	    player2 = new MenuItem(playerMenu, SWT.RADIO);
	    player2.setText(MENU_SETTINGS_MENU_PLAYER_ITEM_PLAYER2);
	    player2.addSelectionListener(new playerListener());
	    if (game.settings.getPlayer() == CGSettings.PLAYER2)	player2.setSelection(true);
	    else													player1.setSelection(true);

	    final MenuItem address = new MenuItem(settingsMenu, SWT.PUSH);
	    address.setText(MENU_SETTINGS_ITEM_ADDRESS);
	    address.addSelectionListener(new addressListener());

		shell.setMenuBar(menuBar);
	}

	// disable/enable the menu bar.
	public void disable() {
		menuBar.setEnabled(false);
	}
	public void enable() {
		menuBar.setEnabled(true);
	}
	
	// disable/enable the 'Undo' item.
	public void disableUndo() {
		undo.setEnabled(false);
	}
	public void enableUndo() {
		undo.setEnabled(true);
	}

	
	
	// the undo item's functionality.
	private class undoListener implements SelectionListener {
		public void widgetSelected(SelectionEvent event) {
			game.crossword.cancelWords(true);
			game.opponentSlots.undo();
			game.crossword.undoOpponent();
    		game.scoreTable.setAlphabetCounter();
    		
    		int minusPts = game.scoreTable.undoPts;
    		game.scoreTable.increaseOpponentScore(-minusPts);
    		
			// if connected to opponent, send the move.
			if (game.connection.connected) {
				CGMove move = new CGMove(game, CGProtocol.MOVE_TYPES_UNDOFIX, minusPts);
				game.connection.sendMove(move);
				game.connection.waitForTurn();
			}
	    }
		public void widgetDefaultSelected(SelectionEvent event) {
			widgetSelected(event);
		}
	}
	
	// the dictionary item's functionality: open up the dictionary dialog.
	private class dictionaryListener implements SelectionListener {
		public void widgetSelected(SelectionEvent event) {
			game.dictionary.open(game.crossword.getInsertedWord());
	    }
		public void widgetDefaultSelected(SelectionEvent event) {
			widgetSelected(event);
		}
	}
	
	// the reset item's functionality: shut down any connections and initialize a new game.
	private class resetListener implements SelectionListener {
		public void widgetSelected(SelectionEvent event) {
			if (game.confirmed())	game.initialize();
	    }
		public void widgetDefaultSelected(SelectionEvent event) {
			widgetSelected(event);
		}
	}
	
	// the exit item's functionality: shut down the main window when selected.
	// exit() needs to be seen by CrosswordGame.
	public void exit() {
		try {
	    	if (game.noMovesMade() && !game.connection.connected)	shell.dispose();
	    	
			if (
				game.confirmed(
						EXIT_CONFIRMATION_TITLE,
						EXIT_CONFIRMATION_MESSAGE_CONNECTED,
						EXIT_CONFIRMATION_MESSAGE
				)
			) shell.dispose();
		}
		
		catch (SWTException ex) {
			;
		}
	}
	private class exitListener implements SelectionListener {
	    public void widgetSelected(SelectionEvent event) {
	    	exit();
	    }
		public void widgetDefaultSelected(SelectionEvent event) {
			widgetSelected(event);
		}
	}
	
	// the player selector functionality.
	private class playerListener implements SelectionListener {
	    public void widgetSelected(SelectionEvent event) {
	    	if ((MenuItem) event.widget == player1) {
	    		game.settings.setPlayer(CGSettings.PLAYER1);
	    	}
	    	else {
	    		game.settings.setPlayer(CGSettings.PLAYER2);
	    	}
	    }
		public void widgetDefaultSelected(SelectionEvent event) {
			widgetSelected(event);
		}
	}

	// the opponent IP address functionality: open the address dialog.
	private class addressListener implements SelectionListener {
	    public void widgetSelected(SelectionEvent event) {
	    	CGAddressDialog dialog = new CGAddressDialog(shell, game);
	    	dialog.open();
	    }
		public void widgetDefaultSelected(SelectionEvent event) {
			widgetSelected(event);
		}
	}
}
